<?php

namespace App\Http\Controllers\user;

use App\Http\Controllers\Controller;
use App\Models\Bank;
use App\Models\BankList;
use App\Models\BonusLedger;
use App\Models\Category;
use App\Models\Deposit;
use App\Models\Notice;
use App\Models\Package;
use App\Models\PaymentMethod;
use App\Models\Purchase;
use App\Models\Rebate;
use App\Models\Task;
use App\Models\User;
use App\Models\UserLedger;
use App\Models\VipSlider;
use App\Models\VipType;
use App\Models\Withdrawal;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Facades\Validator;

class UserController extends Controller
{
    public function dashboard(Request $request)
    {
        $user = User::find(\auth()->user()->id);

        $packages = Package::where(['status' => 'active', 'is_default' => '0'])->get();

        $sliders = VipSlider::where(['page_type' => 'home_page', 'status' => 'active'])->get();

        $rebate = Rebate::first();
        $categorys = Category::where(['status' => 'active'])->get();
        $tasks = Task::where(['status' => 'active'])->take(2)->get();

        return view('app.main.home.index', compact('sliders', 'packages', 'rebate', 'sliders', 'categorys', 'tasks'));
    }

    public function product(Request $request)
    {
        $user = \auth()->user();

        $packages = Package::where(['status' => 'active', 'is_default' => '0'])->get();
        $categorys = Category::where(['status' => 'active'])->get();
        $total_invest = Purchase::where('user_id', $user->id)->sum('amount');
        $total_interest = Purchase::where('user_id', $user->id)->sum('interest');

        return view('app.main.product.index', compact('packages', 'categorys', 'total_invest', 'total_interest'));
    }

    public function vip()
    {
        $vips = VipType::where(['status' => 'active'])->get();
        return view('app.main.vip', compact('vips'));
    }

    public function profile()
    {
        $user = \auth()->user();
        $total_invest = Purchase::where('user_id', $user->id)->sum('amount');
        return view('app.main.mine.index', compact('total_invest'));
    }

    public function update_profile(Request $request)
    {
        $user = User::find(Auth::id());
        $path = uploadImage(false ,$request, 'photo', 'upload/profile/', 200, 200 ,$user->photo);
        $user->photo = $path ?? $user->photo;

        $user->update();
        return redirect()->route('my.profile')->with('success', 'Successfully updated your profile photo');
    }

    public function personal_details()
    {
        return view('app.main.update_personal_details');
    }

    public function personal_details_submit(Request $request)
    {
        $validate = Validator::make($request->all(), [
            'name'=> 'required',
            'methods'=> 'required',
            'number'=> 'required',
        ]);
        if ($validate->fails()){return redirect()->route('user.personal-details')->withErrors($validate->errors());}

        $user = User::find(Auth::id());

        $user->name = $request->name;
        $user->gateway_number = $request->number;
        $user->gateway_method = $request->methods;
        $user->update();
        return redirect()->route('user.personal-details')->with('success', 'Successfully updated your personal details and credentials');
    }

    public function bonus_ledger()
    {
        $datas = BonusLedger::with(['bonus', 'user'])->where('user_id', Auth::id())->orderByDesc('id')->get();
        return view('app.main.bonus.bonus-preview', compact('datas'));
    }

    public function payment_ledger()
    {
        $payments = Deposit::where('user_id', Auth::id())->orderByDesc('id')->get();
        return view('app.main.recharge.payment-preview', compact('payments'));
    }

    public function withdraw_ledger()
    {
        $withdraws = Withdrawal::with('payment_method')->where('user_id', Auth::id())->orderByDesc('id')->get();
        return view('app.main.withdraw.withdraw-preview', compact('withdraws'));
    }

    public function balance_ledger()
    {
        $pending_withdraw = Withdrawal::where('user_id', Auth::id())->where('status', 'pending')->orderByDesc('id')->get()->pluck('amount')->sum();
        $pending_payment = Deposit::where('user_id', Auth::id())->where('status', 'pending')->orderByDesc('id')->get()->pluck('amount')->sum();
        $transactions = UserLedger::where(['user_id' => Auth::id(), 'status' => 'approved'])->where('amount', '!=', '0')->whereIn('reason', ['refer_bonus', 'withdraw_request'])->get();
        return view('app.main.balance_ledger', compact('pending_withdraw', 'pending_payment', 'transactions'));
    }

    public function secret()
    {
        return view('app.main.secret');
    }

    public function password()
    {
        return view('app.main.password');
    }

    public function changepassword(Request $request)
    {
        if (Hash::check($request->current_password, Auth::user()->password)){
            if ($request->new_password == $request->confirm_password)
            {
                User::where('id', Auth::id())->update([
                    'password'=> Hash::make($request->new_password)
                ]);
                return back()->with('error', 'Successfully changed.');
            }else{
                return back()->with('error', 'Confirm password does not match');
            }
        }else{
            return back()->with('error', 'Current password incorrect');
        }
    }

    public function emailVerification($email)
    {
        $email = base64_decode($email);
        $user = User::where('email', $email)->first();
        if ($user){
            return view('app.auth.verify', compact('user'));
        }else{
            return redirect()->route('register')->with('error','Registration failed. please try again');
        }
    }

    public function verified_to_login($user_id, $code)
    {
        $user = User::where('id', $user_id)->first();
        if ($user && $user->code == $code)
        {
            User::where('id', $user->id)->update([
                'email_verified_at'=> Carbon::now()
            ]);

            //Create Purchase record
            $p = Package::where('is_default', '1')->first();
            $model = new Purchase();
            $model->user_id = $user_id;
            $model->package_id = $p ? $p->id : 1;
            $model->amount = $p ? $p->price : 0;
            $model->date = Carbon::now();
            $model->status = 'active';
            $model->save();

            $ledger = new UserLedger();
            $ledger->user_id = $user_id;
            $ledger->reason = 'purchase_package';
            $ledger->perticulation = 'Congratulations already activate default vip for registered in our platform.';
            $ledger->amount = $p ? $p->price : 0;
            $ledger->credit = $p ? $p->price : 0;
            $ledger->status = 'pending';
            $ledger->date = date('d-m-Y H:i');
            $ledger->save();

            Auth::login($user);
            return redirect()->route('dashboard')->with('success', 'You are successfully logged in.');
        }else{
            return redirect()->route('register')->with('error','Registration failed. please try again');
        }
    }

    public function verification_time_out($user_id)
    {
        User::find($user_id)->delete();
        return redirect()->route('register')->with('error','Registration failed. please try again');
    }


    public function notice()
    {
        $datas = Notice::where('status', 'active')->orderByDesc('id')->get();
        return view('app.main.notice', compact('datas'));
    }



    public function card()
    {
        $methods = PaymentMethod::where('status', 'active')->where('id', '!=', 4)->get()->unique('type');
        $bank_list = BankList::where(['status' => 1])->get();

        return view('app.main.gateway_setup', compact('methods', 'bank_list'));
    }



    public function setupGateway(Request $request)
    {
        $this->validate($request, [
            'full_name' => 'required',
            'bank_account' => 'required',
            //'bank_name' => 'required',
            'bank_code' => 'required',
            //'password' => 'required',
            'is_default' => ''
        ]);

        /*if (!\Hash::check($request->password, auth()->user()->password)) {
            return redirect()->back()->with("error", "Invalid login password");
        }*/

        // Find Bank List
        $bank_list = BankList::where(['bank_code' => $request->bank_code, 'status' => 1])->first();

        // Exception
        if(!$bank_list) redirect()->back()->with("error", "Bank Name not available");

        $bank = Bank::where('user_id', auth()->user()->id)->first();
        if (!$bank) {
            $bank = new Bank();
        }
        $bank->full_name = $request->full_name;
        $bank->bank_account = $request->bank_account;
        //$bank->bank_name = $request->bank_name;
        $bank->bank_name = $bank_list->name;
        $bank->bank_code = $request->bank_code;
        $bank->ifsc = $request->bank_code;
        $bank->is_default = ($request->is_default == 'on') ? 1 : 0;
        $bank->user_id = auth()->user()->id;

        if ($bank->save()) {
            return redirect()->back()->with("success", "Bank Details Updated successfully");
        } else {
            return redirect()->back()->with("error", "Something went wrong. bank not update yet.");
        }
    }

    

    public function usdt()
    {
        return view('app.main.crypto_setup');
    }


    public function setupGatewayUsdt(Request $request)
    {
        $this->validate($request, [
            'usdt_name' => 'required',
            'usdt_number' => 'required',
            'password' => 'required'
        ]);

        if (!\Hash::check($request->password, auth()->user()->password)) {
            return redirect()->back()->with("error", "Invalid login password");
        }

        User::where('id',Auth::id())->update([
            'usdt_name'=> $request->usdt_name,
            'usdt_number'=> $request->usdt_number
        ]);
        return redirect()->back()->with('success','USDT update successful.');
    }

    public function invite()
    {
        $vips = Package::where('status', 'active')->get();
        $commissions = UserLedger::where('user_id', Auth::id())
            ->where('reason', '!=', 'vip_request')
            ->where('reason', '!=', 'purchase_package')
            ->orderBy('created_at', 'desc')
            ->get();

        return view('app.main.invite', compact('vips', 'commissions'));
    }

    public function help_center()
    {
        return view('app.main.help_center');
    }


    public function about()
    {
        return view('app.main.about-us');
    }

    public function invite_bonus()
    {
        $vips = Package::where('status', 'active')->get();
        return view('app.main.invite_bonus', compact('vips'));
    }

    public function whatsapp_link()
    {
        return view('app.main.whatsapp_link');
    }

    public function telegram_link()
    {
        return view('app.main.telegram_link');
    }

    public function download_apk(){
        $file= public_path('hscsv.apk');
        return response()->file($file ,[
            'Content-Type'=>'application/vnd.android.package-archive',
            'Content-Disposition'=> 'attachment; filename="ehaapp.apk"',
        ]) ;
    }


    public function share()
    {
        $rebate = Rebate::first();
        return view('app.main.share', compact('rebate'));
    }


}
