<?php

namespace App\Http\Controllers\user;

use App\Http\Component\AppController;
use App\Http\Controllers\Controller;
use App\Http\Services\PaymentServices;
use App\Http\Services\WalletServices;
use App\Models\PaymentMethod;
use App\Models\Purchase;
use App\Models\Setting;
use App\Models\User;
use App\Models\UserLedger;
use App\Models\Withdrawal;
use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Mail;
use Illuminate\Support\Facades\Validator;

class WithdrawController extends AppController
{
    public function withdraw()
    {
        // Check if Bank Added
        if(!auth()->user()->bankAccount) {
            return redirect()->route('user.card')->with("error", "Please update your bank account first");
        }

        return view('app.main.withdraw.index');

        $userMethod = PaymentMethod::where('id', Auth::user()->gateway_method)->first();
        $methods = PaymentMethod::where('status', 'active')->get()->unique('type');
        return view('app.main.withdraw.index', compact('methods', 'userMethod'));
    }

    public function withdrawRequest(Request $request)
    {
        $validate = Validator::make($request->all(), [
            'number' => 'required|numeric',
            'pay_method' => 'required|numeric',
            'amount' => 'required|numeric'
        ]);

        // if (Carbon::now()->isSunday() || Carbon::now()->isSaturday()){
        //     return response()->json(['status' => false, 'message' => 'You can not withdraw because today is off day']);
        // }

        //Check withdraw pending
        $w = Withdrawal::where('user_id', \auth()->user()->id)->where('status', 'pending')->orderByDesc('id')->first();
        if ($w)
        {
            return response()->json(['status' => false, 'message' => 'Already your withdraw is pending. you do not withdraw before recent pending withdraw approved']);
        }

        if ($validate->fails()) {
            return response()->json(['status' => false, 'errors' => $validate->errors()]);
        }
        
        $start = 10; //Eg. start hour 10PM
        $end = 22;  //Eg. end hour 5AM
        $now = Carbon::now('Asia/Dhaka');


        //Check is user premium member or not
        $premium = Purchase::where('user_id', Auth::id())->where('package_id', '>', 1)->get();
        
        if($start <= $now->hour && $end > $now->hour){
  
            if ($premium->count() > 0){
            $payment = PaymentMethod::where('id', $request->pay_method)->first();
            $user = Auth::user();
            if ($payment) {
                if ($request->amount <= $user->balance) {
                    if ($request->amount >= $payment->minimum_withdraw) {
                        if ($request->amount <= $payment->maximum_withdraw) {
                            $requestData = $request->all();
                            $data = ['number'=> $request->number, 'method'=> $request->pay_method,'code'=> rand(000000, 999999)];
                            Mail::send('mail.verify_code', compact('data'), function ($m) use ($user) {
                                $m->from(env('MAIL_USERNAME'), config('app.name', 'APP Name'));
                                $m->to($user->email, $user->name)->subject('Email verification');
                            });

                            session()->put('withdraw_data', array_merge($requestData, ['code'=>$data['code']]));

                            return response()->json(['status' => true]);
                        } else {
                            return response()->json(['status' => false, 'message' => "Minimum withdraw $payment->maximum_withdraw"]);
                        }
                    } else {
                        return response()->json(['status' => false, 'message' => 'Withdraw amount must be greater then ' . $payment->minimum_withdraw]);
                    }
                } else {
                    return response()->json(['status' => false, 'message' => 'Your sufficient balance is low.']);
                }
            } else {
                return response()->json(['status' => false, 'message' => 'Withdraw method not fount.']);
            }
        }else{
            return response()->json(['status' => false, 'message' => 'You are not a premium member. please invest our platform.']);
        }
        }else{
            return response()->json(['status' => false, 'message' => 'Withdraw time 10AM to 5PM']);
        }
    }

    public function withdrawConfirm()
    {
        $data = session()->get('withdraw_data');
        $method = PaymentMethod::find($data['pay_method']);
        return view('app.main.withdraw.confirm', compact('data', 'method'));
    }

    public function usdt_withdraw()
    {
        return view('app.main.withdraw.usdt');
    }

    public function withdrawConfirmSubmit(Request $request, PaymentServices $payment, WalletServices $wallet)
    {
        try {
            $validate = Validator::make($request->all(), [
                'amount' => 'required|numeric',
                'password' => 'required|string'
            ]);
            if ($validate->fails()) {
                return back()->with('error', $validate->errors());
                return response()->json(['status' => false, 'errors' => $validate->errors()]);
            }
    
            if (!\Hash::check($request->password, auth()->user()->password)) throw new \Exception("Invalid login password");

            $user = User::find(\auth()->user()->id);
            $setting = Setting::first();

            // Withdraw Suspended
            if($setting->open_payout != 1 || $user->open_payout != 1) throw new \Exception("Sorry! service not available. Try again later");

            if ($request->amount <= setting('minimum_withdraw')) throw new \Exception('Minimum Withdraw ' . setting('minimum_withdraw'));
            
            if (!$request->amount >= setting('maximum_withdraw')) throw new \Exception('Maximum Withdraw ' . setting('maximum_withdraw'));

            // Find Invest
            $hasInvest = Purchase::where(['user_id' => $user->id])->first();

            // Verify Invest
            if(!$hasInvest) throw new \Exception("Sorry! before placed a withdraw, you need to have any active product in your account.");
            
            
            $start = 9; //Eg. start hour 10PM
            $end = 21;  //Eg. end hour 5AM
            $now = Carbon::now();

            // Verify Withdraw Time
            //if(!$start <= $now->hour && !$end > $now->hour) throw new \Exception('Withdraw allowed 11AM to 9PM');

            $status_id = 'pending';
            $status_text = 'Your withdraw request status is pending please wait for admin approval or communication with us.';
            $reference = rand(00000,99999);

            $userBalance = $user->income;
            $pay_amount = $request->amount;
            $currency = $setting->cur_text;

            // Low Balance
            if($userBalance < $pay_amount){
                return $this->handleResponseRedirect($request, 'You do not have sufficient balance to pay. Please recharge your wallet', 401, null, 'user.onepay', ['currency' => $currency]);
            }

            // Debit Balance
            $debitBalance = $wallet->debitAvailableBalance($user, 2, $currency, $pay_amount);

            // Exception
            if($debitBalance instanceOf \Exception) {
                return $this->handleResponseRedirect($request, $debitBalance->getMessage(), 401, null, 'user.withdraw', ['currency' => $currency]);
            }
            
            $charge = 0;
            if (setting('withdraw_charge') > 0) {
                $charge = ($request->amount * setting('withdraw_charge')) / 100;
            }

            $finalAmount = $request->amount - $charge;

            // Auto Payout
            if($setting->auto_transfer) {

                // Transfer Payment
                $transferPayment = $payment->payout($reference, $currency, $finalAmount, $setting->auto_transfer_default, $user->bankAccount->bank_code, $user->bankAccount->bank_account, $user->bankAccount->full_name);

                // Exception
                if($transferPayment['status'] == false) {

                    // Refund Payment
                    $wallet->creditAvailableBalance($user, 2, $currency, $request->amount);

                    throw new \Exception($transferPayment['message']);
                    
                }

                $status_id = 'approved';
                $status_text = 'Your withdraw request has been approved';
            }

            $account_info = [
                'bank_account' => $user->bankAccount->bank_account,
                'full_name' => $user->bankAccount->full_name,
                'bank_name' => $user->bankAccount->bank_name,
                'bank_code' => $user->bankAccount->bank_code,
            ];

            // Withdraw
            $withdrawal = new Withdrawal();
            $withdrawal->payment_method_id = Auth::user()->gateway_method;
            $withdrawal->user_id = $user->id;
            $withdrawal->trx = $reference;
            $withdrawal->account_info = json_encode($account_info);
            $withdrawal->number = $user->bankAccount->bank_account;
            $withdrawal->amount = $request->amount;
            $withdrawal->currency = $currency;
            $withdrawal->charge = $charge;
            $withdrawal->final_amount = $finalAmount;
            $withdrawal->status = $status_id;

            // User Ledger
            if ($withdrawal->save()){
                $ledger = new UserLedger();
                $ledger->user_id = $user->id;
                $ledger->reason = 'withdraw_request';
                $ledger->perticulation = $status_text;
                $ledger->amount = $request->amount;
                $ledger->debit = $finalAmount;
                $ledger->status = $status_id;
                $ledger->date = date('d-m-Y H:i');
                $ledger->save();
            }
            
            return $this->handleResponseRedirect($request, 'Your Withdraw request has been submitted', 200, null, 'user.withdraw');
            
        } catch (\Exception $th) {
            //throw $th;
            return $this->handleResponseRedirect($request, $th->getMessage(), 401, null, 'user.withdraw');
        }
        
    }


    public function withdrawConfirmSubmitUSDT(Request $request)
    {
        $validate = Validator::make($request->all(), [
            'amount' => 'required|numeric'
        ]);
        if ($validate->fails()) {
            return back()->with('error', $validate->errors());
            return response()->json(['status' => false, 'errors' => $validate->errors()]);
        }

        if (!Auth::user()->usdt_number || !Auth::user()->usdt_name){
            return back()->with('success', 'Please setup your bank');
        }

        if (!\Hash::check($request->password, auth()->user()->password)) {
            return back()->with('success', 'Invalid login password');
        }

        $user = Auth::user();

        if ($request->amount <= $user->balance) {
            if (($request->amount / setting('rate')) >= setting('minimum_withdraw_usdt')) {
                if (($request->amount / setting('rate')) <= setting('maximum_withdraw_usdt')) {
                    $charge = 0;
                    if (setting('withdraw_charge') > 0) {
                        $charge = setting('withdraw_charge_usdt');
                    }

                    //Update User Balance
                    $balance = $user->balance - $request->amount;
                    User::where('id', $user->id)->update([
                        'balance'=> $balance
                    ]);

                    $reference = rand(00000,99999);

                    //Withdraw
                    $withdrawal = new Withdrawal();
                    $withdrawal->payment_method_id = Auth::user()->gateway_method;
                    $withdrawal->user_id = $user->id;
                    $withdrawal->number = Auth::user()->usdt_number;
                    $withdrawal->trx = $reference;
                    $withdrawal->amount = $request->amount;
                    $withdrawal->usdt = ($request->amount / setting('rate')) - $charge;
                    $withdrawal->currency = 'Bangladesh';
                    $withdrawal->charge = $charge;
                    $withdrawal->final_amount = 0;
                    $withdrawal->status = 'pending';

                    //User Ledger
                    if ($withdrawal->save()){
                        $ledger = new UserLedger();
                        $ledger->user_id = $user->id;
                        $ledger->reason = 'withdraw_request';
                        $ledger->perticulation = 'Your withdraw request status is pending please wait for admin approval or communication with us.';
                        $ledger->amount = $request->amount;
                        $ledger->is_usdt = 'yes';
                        $ledger->debit = $request->amount - $charge;
                        $ledger->status = 'pending';
                        $ledger->date = date('d-m-Y H:i');
                        $ledger->save();
                    }
                    return back()->with('success', 'Withdraw request send successfully. Please wait for admin approval.');
                    return response()->json(['status' => true, 'message' => 'Withdraw request send successfully. Please wait for admin approval.']);
                } else {
                    return back()->with('error', 'Withdraw amount must be less then ' . setting('maximum_withdraw'));
                    return response()->json(['status' => false, 'message' => 'Withdraw amount must be less then ' . setting('maximum_withdraw_usdt')]);
                }
            } else {
                return back()->with('error', 'Withdraw amount must be greater then ' . setting('maximum_withdraw'));
                return response()->json(['status' => false, 'message' => 'Withdraw amount must be greater then ' . setting('minimum_withdraw_usdt')]);
            }
        } else {
            return back()->with('error', 'Your sufficient balance is low.');
            return response()->json(['status' => false, 'message' => 'Your sufficient balance is low.']);
        }
    }

    public function withdrawPreview()
    {
        $withdraws = Withdrawal::with('payment_method')->where('user_id', Auth::id())->orderByDesc('id')->get();
        return view('app.main.withdraw.withdraw-preview', compact('withdraws'));
    }
}
