<?php
namespace App\Http\Services\Payout\Bank;

use App\Http\Services\Payout\Bank\ShpayBankPayoutServices;
use App\Models\Setting;

/**
 * Laravel/Symfony Developer
 * Name: Muongozo
 * Telegram: @muongozotrader
 * Hire me via Telegram: muongozotrader
 */
class ProcessBankPayoutServices
{
    private $manual;
    private $shpay;
    private $gtr;

    public function __construct(
        ManualBankPayoutServices $manual,
        ShpayBankPayoutServices $shpay,
        GTRBankPayoutServices $gtr
    )
    {
        $this->manual = $manual;
        $this->shpay = $shpay;
        $this->gtr = $gtr;
    }

    /**
     * Transfer Payment
     * 
     * @param string reference
     * @param string currency
     * @param string amount
     * @param string method
     * @param string bank code
     * @param string bank name
     * @param string account number
     * @param string account name
     * @param string narration
     */
    public function transfer(
        string $reference,
        string $currency,
        string $amount,
        string $method,
        string $bank_code,
        string $bank_name = null,
        string $account_number,
        string $account_name,
        string $narration = null
    )
    {
        try {

            // Hold User
            $setting = Setting::first();

            // Verify transfer Type
            if(!in_array($setting->auto_transfer_default, ['shpay', 'gtr'])) throw new \Exception("Payout not available at the moment");
                
            // Shpay
            if(in_array($setting->auto_transfer_default, ['shpay'])) {
                $payment = $this->shpay->transfer($reference, $currency, $amount, $method, $bank_code, $bank_name, $account_number, $account_name, $narration);
            }

            // GTR
            if(in_array($setting->auto_transfer_default, ['gtr'])) {
                $payment = $this->gtr->transfer($reference, $currency, $amount, $method, $bank_code, $bank_name, $account_number, $account_name, $narration);
            }

            // Exception
            if($payment instanceof \Exception) throw new \Exception($payment->getMessage());

            // Response
            return $payment;

        } catch (\Exception $th) {
            //throw $th;
            return $th;
        }
    }
}
